@@ -144,6 +144,11 @@ typedef struct {
 		sfree (data); \
 	} while (0)
 
+/*
+ * Public variable
+ */
+extern char **environ;
+
 /*
  * private variables
  */
@@ -194,6 +199,8 @@ struct {
 	{ "Collectd::TYPE_DATASET",       PLUGIN_DATASET },
 	{ "Collectd::DS_TYPE_COUNTER",    DS_TYPE_COUNTER },
 	{ "Collectd::DS_TYPE_GAUGE",      DS_TYPE_GAUGE },
+	{ "Collectd::DS_TYPE_DERIVE",     DS_TYPE_DERIVE },
+	{ "Collectd::DS_TYPE_ABSOLUTE",   DS_TYPE_ABSOLUTE },
 	{ "Collectd::LOG_ERR",            LOG_ERR },
 	{ "Collectd::LOG_WARNING",        LOG_WARNING },
 	{ "Collectd::LOG_NOTICE",         LOG_NOTICE },
@@ -267,7 +274,10 @@ static int hv2data_source (pTHX_ HV *hash, data_source_t *ds)
 	if (NULL != (tmp = hv_fetch (hash, "type", 4, 0))) {
 		ds->type = SvIV (*tmp);
 
-		if ((DS_TYPE_COUNTER != ds->type) && (DS_TYPE_GAUGE != ds->type)) {
+		if ((DS_TYPE_COUNTER != ds->type)
+				&& (DS_TYPE_GAUGE != ds->type)
+				&& (DS_TYPE_DERIVE != ds->type)
+				&& (DS_TYPE_ABSOLUTE != ds->type)) {
 			log_err ("hv2data_source: Invalid DS type.");
 			return -1;
 		}
@@ -320,8 +330,12 @@ static int av2value (pTHX_ char *name, AV *array, value_t *value, int len)
 		if (NULL != tmp) {
 			if (DS_TYPE_COUNTER == ds->ds[i].type)
 				value[i].counter = SvIV (*tmp);
-			else
+			else if (DS_TYPE_GAUGE == ds->ds[i].type)
 				value[i].gauge = SvNV (*tmp);
+			else if (DS_TYPE_DERIVE == ds->ds[i].type)
+				value[i].derive = SvIV (*tmp);
+			else if (DS_TYPE_ABSOLUTE == ds->ds[i].type)
+				value[i].absolute = SvIV (*tmp);
 		}
 		else {
 			return -1;
@@ -637,8 +651,12 @@ static int value_list2hv (pTHX_ value_list_t *vl, data_set_t *ds, HV *hash)
 
 		if (DS_TYPE_COUNTER == ds->ds[i].type)
 			val = newSViv (vl->values[i].counter);
-		else
+		else if (DS_TYPE_GAUGE == ds->ds[i].type)
 			val = newSVnv (vl->values[i].gauge);
+		else if (DS_TYPE_DERIVE == ds->ds[i].type)
+			val = newSViv (vl->values[i].derive);
+		else if (DS_TYPE_ABSOLUTE == ds->ds[i].type)
+			val = newSViv (vl->values[i].absolute);
 
 		if (NULL == av_store (values, i, val)) {
 			av_undef (values);
