@@ -10,4 +10,8 @@ fi
 
 VERSION="`echo \"$VERSION\" | sed -e 's/-/./g'`"
 
-echo -n "$VERSION"
+if test "x`uname -s`" = "xAIX" ; then 
+	echo "$VERSION\c"
+else 
+	echo -n "$VERSION"
+fi
